// Only what is in this header is exposed from the
// wrapping DLL.
// The goal is not exposed anything from the managed world.
// So expose only interfaces (abstract C++ classes) and their
// factory methods for creation and destruction.


#pragma once

#ifdef WRAPPER_EXPORT
#define WRAPPER_API __declspec(dllexport)
#else
#define WRAPPER_API __declspec(dllimport)
#endif

// Example of an interface declaring the communication
// functionality.
class IClient
{
public:
    virtual ~IClient(void){}

    virtual void OpenConnection() = 0;
    virtual void CloseConnection() = 0;
    virtual int SendCalculateRequest(int a, int b) = 0;
};

// And here are factories for creating and destructing.
extern "C" WRAPPER_API IClient* CreateClient(const wchar_t* address);
extern "C" WRAPPER_API void DestroyClient(IClient* client);

